declare module "vendor/microevent" {
    class MicroEvent<EventMap extends Record<string, any[]> = Record<string, any[]>, N extends keyof EventMap = keyof EventMap> {
        _eventTypes: EventMap;
        _events: Partial<Record<N, ((...args: EventMap[N]) => void)[]>>;
        bind<EN extends N>(eventName: EN, callback: (...args: EventMap[EN]) => void): void;
        unbind<EN extends N>(eventName: EN, callback: (...args: EventMap[EN]) => void): void;
        trigger<EN extends N>(eventName: EN, ...args: EventMap[EN]): void;
    }
    type TypeArg<C extends MicroEvent> = C extends MicroEvent<infer U> ? U : never;
    export type EventArg<C extends MicroEvent, N extends keyof TypeArg<C>> = TypeArg<C>[N][0];
    export default MicroEvent;
}
declare module "lib/verso-browser/animation" {
    export default class Animation {
        run: number;
        el: HTMLElement;
        constructor(el: HTMLElement);
        animate({ x, y, scale, easing, duration }?: {
            x: string | number;
            y?: string | number;
            scale?: number;
            easing?: string;
            duration?: number;
        }, callback?: () => void): this;
    }
}
declare module "lib/verso-browser/page_spread" {
    import './page_spread.styl';
    interface PageSpreadInit {
        id: string;
        type?: string;
        pageIds: string[];
        width: number;
        left: number;
        maxZoomScale: number;
    }
    export default class PageSpread {
        visibility: string;
        positioned: boolean;
        active: boolean;
        el: HTMLElement;
        options: PageSpreadInit;
        id: string;
        type?: string;
        pageIds: string[];
        width: number;
        left: number;
        maxZoomScale: number;
        constructor(el: HTMLElement, options: PageSpreadInit);
        isZoomable(): boolean;
        isScrollable(): boolean;
        getEl(): HTMLElement;
        getOverlayEls(): NodeListOf<HTMLElement>;
        getPageEls(): NodeListOf<HTMLElement>;
        getRect(): DOMRect;
        getContentRect(): {
            top: number;
            left: number;
            right: number;
            bottom: number;
            width: number;
            height: number;
        };
        getId(): string;
        getType(): string;
        getPageIds(): string[];
        getWidth(): number;
        getLeft(): number;
        getMaxZoomScale(): number;
        getVisibility(): string;
        setVisibility(visibility: any): this;
        position(): this;
        activate(): void;
        deactivate(): void;
    }
}
declare module "verso-browser" {
    import MicroEvent from "vendor/microevent";
    import Animation from "lib/verso-browser/animation";
    import PageSpread from "lib/verso-browser/page_spread";
    import Manager from './vendor/hammer/Manager';
    import './verso.styl';
    interface VersoInit {
        swipeVelocity?: number;
        swipeThreshold?: number;
        navigationDuration?: number;
        navigationPanDuration?: number;
        zoomDuration?: number;
        doubleTapDelay?: number;
        pageId?: string;
    }
    export default class Verso extends MicroEvent {
        position: number;
        pinching: boolean;
        panning: boolean;
        transform: {
            left: number;
            top: number;
            scale: number;
        };
        startTransform: {
            left: number;
            top: number;
            scale: number;
        };
        started: boolean;
        destroyed: boolean;
        _events: {};
        el: HTMLElement;
        scrollerEl: HTMLElement;
        pageSpreadEls: NodeListOf<HTMLElement>;
        pageSpreads: PageSpread[];
        pageIds: Record<PageSpread['pageIds'][number], PageSpread>;
        options: VersoInit;
        swipeVelocity: number;
        swipeThreshold: number;
        navigationDuration: number;
        navigationPanDuration: number;
        zoomDuration: number;
        tap: {
            count: number;
            delay: number;
            timeout?: NodeJS.Timeout;
        };
        animation: Animation;
        hammer: Manager;
        coarsePointerQuery: MediaQueryList | null;
        prefersReducedMotionQuery: MediaQueryList | null;
        constructor(el: HTMLElement, options?: VersoInit);
        start(): this;
        destroy(): void | this;
        first(options?: Parameters<typeof this.navigateTo>[1]): void;
        prev(options?: Parameters<typeof this.navigateTo>[1]): void;
        next(options?: Parameters<typeof this.navigateTo>[1]): void;
        last(options?: Parameters<typeof this.navigateTo>[1]): void;
        navigateTo(newPosition: number, options?: {
            duration?: number;
            velocity?: number;
        }): void;
        getPosition(): number;
        setPosition(position: number): this;
        getLeftTransformFromPageSpread(position: number, pageSpread: PageSpread): number;
        getCarouselFromPageSpread(pageSpreadSubject: PageSpread): {
            visible: PageSpread[];
            gone: PageSpread[];
        };
        getCoordinateInfo(x: number, y: number, pageSpread: PageSpread): {
            x: number;
            y: number;
            contentX: number;
            contentY: number;
            pageX: number;
            pageY: number;
            overlayEls: HTMLElement[];
            pageEl: null | HTMLElement;
            isInsideContentX: boolean;
            isInsideContentY: boolean;
            isInsideContent: boolean;
        };
        getPageSpreadCount(): number;
        getActivePageSpread(): PageSpread;
        getPageSpreadFromPosition(position: number): PageSpread;
        getPageSpreadPositionFromPageId(pageId: string): number;
        zoomTo({ duration, easing, scale, x, y, bounds }?: {
            duration?: number;
            easing?: string;
            scale: number;
            x: number;
            y: number;
            bounds?: boolean;
        }, callback?: () => void): void;
        refresh(): this;
        onPanStart: (e: any) => void;
        onPanMove: (e: any) => void;
        onPanEnd: (e: any) => void;
        onPinchStart: () => void;
        onPinchMove: (e: any) => void;
        onPinchEnd: (e: any) => void;
        onPress: (e: any) => void;
        onContextmenu: (e: any) => boolean;
        onWheel: (e: WheelEvent) => void;
        onSingletap: (e: any) => void;
        onTouchStart: (e: any) => void;
        onTouchEnd: (e: any) => void;
        onResize: () => void;
    }
}
