var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);

// vendor/microevent.ts
var MicroEvent = class {
  constructor() {
    __publicField(this, "_eventTypes");
    __publicField(this, "_events", {});
  }
  bind(eventName, callback) {
    var _a2;
    const callbacks = (_a2 = this._events[eventName]) != null ? _a2 : [];
    this._events[eventName] = [...callbacks, callback];
  }
  unbind(eventName, callback) {
    var _a2;
    const callbacks = (_a2 = this._events[eventName]) != null ? _a2 : [];
    this._events[eventName] = callbacks.filter((cb) => cb !== callback);
  }
  trigger(eventName, ...args) {
    var _a2;
    const callbacks = (_a2 = this._events[eventName]) != null ? _a2 : [];
    for (const callback of callbacks) callback(...args);
  }
};
var microevent_default = MicroEvent;

// lib/verso-browser/animation.ts
var Animation = class {
  constructor(el) {
    __publicField(this, "run", 0);
    __publicField(this, "el");
    this.el = el;
  }
  animate({
    x = 0,
    y = 0,
    scale = 1,
    easing = "ease-out",
    duration = 0
  } = {}, callback) {
    const run = ++this.run;
    const transform = "translateX(".concat(x, ") translateY(").concat(y, ") scale(").concat(scale, ")");
    if (this.el.style.transform === transform) {
      callback == null ? void 0 : callback();
    } else if (duration > 0) {
      const transitionEnd = () => {
        if (run !== this.run) return;
        this.el.removeEventListener("transitionend", transitionEnd);
        this.el.style.transition = "none";
        callback == null ? void 0 : callback();
      };
      this.el.addEventListener("transitionend", transitionEnd, false);
      this.el.style.transition = "transform ".concat(easing, " ").concat(duration, "ms");
      this.el.style.transform = transform;
    } else {
      this.el.style.transition = "none";
      this.el.style.transform = transform;
      callback == null ? void 0 : callback();
    }
    return this;
  }
};

// lib/verso-browser/page_spread.ts
var PageSpread = class {
  constructor(el, options) {
    __publicField(this, "visibility", "gone");
    __publicField(this, "positioned", false);
    __publicField(this, "active", false);
    __publicField(this, "el");
    __publicField(this, "options");
    __publicField(this, "id");
    __publicField(this, "type");
    __publicField(this, "pageIds");
    __publicField(this, "width");
    __publicField(this, "left");
    __publicField(this, "maxZoomScale");
    this.el = el;
    this.options = options;
    this.id = this.options.id;
    this.type = this.options.type;
    this.pageIds = this.options.pageIds;
    this.width = this.options.width;
    this.left = this.options.left;
    this.maxZoomScale = this.options.maxZoomScale;
  }
  isZoomable() {
    return this.getMaxZoomScale() > 1 && this.getEl().dataset.zoomable !== "false";
  }
  isScrollable() {
    return this.getEl().classList.contains("verso--scrollable");
  }
  getEl() {
    return this.el;
  }
  getOverlayEls() {
    return this.getEl().querySelectorAll(".verso__overlay");
  }
  getPageEls() {
    return this.getEl().querySelectorAll(".verso__page");
  }
  getRect() {
    return this.getEl().getBoundingClientRect();
  }
  getContentRect() {
    var _a2, _b, _c, _d;
    const rect = {
      top: null,
      left: null,
      right: null,
      bottom: null,
      width: null,
      height: null
    };
    const pageEls = this.getPageEls();
    for (let idx = 0; idx < pageEls.length; idx++) {
      const pageEl = pageEls[idx];
      const pageRect = pageEl.getBoundingClientRect();
      if (rect.top == null || pageRect.top < rect.top) {
        rect.top = pageRect.top;
      }
      if (rect.left == null || pageRect.left < rect.left) {
        rect.left = pageRect.left;
      }
      if (rect.right == null || pageRect.right > rect.right) {
        rect.right = pageRect.right;
      }
      if (rect.bottom == null || pageRect.bottom > rect.bottom) {
        rect.bottom = pageRect.bottom;
      }
    }
    rect.top = (_a2 = rect.top) != null ? _a2 : 0;
    rect.left = (_b = rect.left) != null ? _b : 0;
    rect.right = (_c = rect.right) != null ? _c : 0;
    rect.bottom = (_d = rect.bottom) != null ? _d : 0;
    rect.width = rect.right - rect.left;
    rect.height = rect.bottom - rect.top;
    return rect;
  }
  getId() {
    return this.id;
  }
  getType() {
    return this.type;
  }
  getPageIds() {
    return this.pageIds;
  }
  getWidth() {
    return this.width;
  }
  getLeft() {
    return this.left;
  }
  getMaxZoomScale() {
    return this.maxZoomScale;
  }
  getVisibility() {
    return this.visibility;
  }
  setVisibility(visibility) {
    if (this.visibility !== visibility) {
      this.getEl().style.display = visibility === "visible" ? "block" : "none";
      this.visibility = visibility;
    }
    return this;
  }
  position() {
    if (!this.positioned) {
      this.getEl().style.left = "".concat(this.getLeft(), "%");
      this.positioned = true;
    }
    return this;
  }
  activate() {
    this.active = true;
    this.getEl().dataset.active = this.active;
  }
  deactivate() {
    this.active = false;
    this.getEl().dataset.active = this.active;
  }
};

// lib/verso-browser/vendor/hammer/Input.js
var INPUT_TYPE_TOUCH = "touch";
var INPUT_TYPE_MOUSE = "mouse";
var COMPUTE_INTERVAL = 25;
var INPUT_START = 1;
var INPUT_MOVE = 2;
var INPUT_END = 4;
var INPUT_CANCEL = 8;
var DIRECTION_NONE = 1;
var DIRECTION_LEFT = 2;
var DIRECTION_RIGHT = 4;
var DIRECTION_UP = 8;
var DIRECTION_DOWN = 16;
var DIRECTION_HORIZONTAL = DIRECTION_LEFT | DIRECTION_RIGHT;
var DIRECTION_VERTICAL = DIRECTION_UP | DIRECTION_DOWN;
var DIRECTION_ALL = DIRECTION_HORIZONTAL | DIRECTION_VERTICAL;
var PROPS_XY = ["x", "y"];
var PROPS_CLIENT_XY = ["clientX", "clientY"];
var TOUCH_INPUT_MAP = {
  touchstart: INPUT_START,
  touchmove: INPUT_MOVE,
  touchend: INPUT_END,
  touchcancel: INPUT_CANCEL
};
function getDistance(p1, p2, props = PROPS_XY) {
  const x = p2[props[0]] - p1[props[0]];
  const y = p2[props[1]] - p1[props[1]];
  return Math.sqrt(x * x + y * y);
}
function directionStr(direction) {
  if (direction === DIRECTION_DOWN) return "down";
  if (direction === DIRECTION_UP) return "up";
  if (direction === DIRECTION_LEFT) return "left";
  if (direction === DIRECTION_RIGHT) return "right";
  return "";
}
function getAngle(p1, p2, props = PROPS_XY) {
  const x = p2[props[0]] - p1[props[0]];
  const y = p2[props[1]] - p1[props[1]];
  return Math.atan2(y, x) * 180 / Math.PI;
}
function getCenter(pointers) {
  let x = 0;
  let y = 0;
  pointers.forEach((pointer) => {
    x += pointer.clientX;
    y += pointer.clientY;
  });
  return {
    x: Math.round(x / pointers.length),
    y: Math.round(y / pointers.length)
  };
}
var getRotation = (start, end) => getAngle(end[1], end[0], PROPS_CLIENT_XY) + getAngle(start[1], start[0], PROPS_CLIENT_XY);
var getScale = (start, end) => getDistance(end[0], end[1], PROPS_CLIENT_XY) / getDistance(start[0], start[1], PROPS_CLIENT_XY);
var getDirection = (x, y) => x === y ? DIRECTION_NONE : Math.abs(x) >= Math.abs(y) ? x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT : y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
var getVelocity = (deltaTime, x, y) => ({
  x: x / deltaTime || 0,
  y: y / deltaTime || 0
});
function computeIntervalInputData(session, input) {
  const last = session.lastInterval || input;
  const deltaTime = input.timeStamp - last.timeStamp;
  let velocity;
  let velocityX;
  let velocityY;
  let direction;
  if (input.eventType !== INPUT_CANCEL && (deltaTime > COMPUTE_INTERVAL || last.velocity === void 0)) {
    const deltaX = input.deltaX - last.deltaX;
    const deltaY = input.deltaY - last.deltaY;
    const v = getVelocity(deltaTime, deltaX, deltaY);
    velocityX = v.x;
    velocityY = v.y;
    velocity = Math.abs(v.x) > Math.abs(v.y) ? v.x : v.y;
    direction = getDirection(deltaX, deltaY);
    session.lastInterval = input;
  } else {
    velocity = last.velocity;
    velocityX = last.velocityX;
    velocityY = last.velocityY;
    direction = last.direction;
  }
  input.velocity = velocity;
  input.velocityX = velocityX;
  input.velocityY = velocityY;
  input.direction = direction;
}
function computeDeltaXY(session, input) {
  const { center } = input;
  let offset = session.offsetDelta || {};
  let prevDelta = session.prevDelta || {};
  const prevInput = session.prevInput || {};
  if (input.eventType === INPUT_START || prevInput.eventType === INPUT_END) {
    prevDelta = session.prevDelta = {
      x: prevInput.deltaX || 0,
      y: prevInput.deltaY || 0
    };
    offset = session.offsetDelta = {
      x: center.x,
      y: center.y
    };
  }
  input.deltaX = prevDelta.x + (center.x - offset.x);
  input.deltaY = prevDelta.y + (center.y - offset.y);
}
function simpleCloneInputData(input) {
  const pointers = input.pointers.map(({ clientX, clientY }) => ({
    clientX: Math.round(clientX),
    clientY: Math.round(clientY)
  }));
  return {
    timeStamp: Date.now(),
    pointers,
    center: getCenter(pointers),
    deltaX: input.deltaX,
    deltaY: input.deltaY
  };
}
function computeInputData(manager, input) {
  const { session } = manager;
  const { pointers } = input;
  const { length: pointersLength } = pointers;
  if (!session.firstInput) session.firstInput = simpleCloneInputData(input);
  if (pointersLength > 1 && !session.firstMultiple) {
    session.firstMultiple = simpleCloneInputData(input);
  } else if (pointersLength === 1) {
    session.firstMultiple = false;
  }
  const { firstInput, firstMultiple } = session;
  const offsetCenter = firstMultiple ? firstMultiple.center : firstInput.center;
  input.center = getCenter(pointers);
  input.timeStamp = Date.now();
  input.deltaTime = input.timeStamp - firstInput.timeStamp;
  input.angle = getAngle(offsetCenter, input.center);
  input.distance = getDistance(offsetCenter, input.center);
  computeDeltaXY(session, input);
  input.offsetDirection = getDirection(input.deltaX, input.deltaY);
  const overallVelocity = getVelocity(
    input.deltaTime,
    input.deltaX,
    input.deltaY
  );
  input.overallVelocityX = overallVelocity.x;
  input.overallVelocityY = overallVelocity.y;
  input.overallVelocity = Math.abs(overallVelocity.x) > Math.abs(overallVelocity.y) ? overallVelocity.x : overallVelocity.y;
  input.scale = firstMultiple ? getScale(firstMultiple.pointers, pointers) : 1;
  input.rotation = firstMultiple ? getRotation(firstMultiple.pointers, pointers) : 0;
  input.maxPointers = !session.prevInput ? input.pointers.length : input.pointers.length > session.prevInput.maxPointers ? input.pointers.length : session.prevInput.maxPointers;
  computeIntervalInputData(session, input);
  input.target = manager.element.contains(input.srcEvent.target) ? input.srcEvent.target : manager.element;
}
function inputHandler(manager, eventType, input) {
  const pointersLen = input.pointers.length;
  const changedPointersLen = input.changedPointers.length;
  const isFirst = eventType & INPUT_START && pointersLen - changedPointersLen === 0;
  const isFinal = eventType & (INPUT_END | INPUT_CANCEL) && pointersLen - changedPointersLen === 0;
  input.isFirst = Boolean(isFirst);
  input.isFinal = Boolean(isFinal);
  if (isFirst) manager.session = {};
  input.eventType = eventType;
  computeInputData(manager, input);
  manager.recognize(input);
  manager.session.prevInput = input;
}
var addEventListeners = (target, types, handler) => types.trim().split(/\s+/g).forEach((type) => {
  target.addEventListener(type, handler, false);
});
var removeEventListeners = (target, types, handler) => types.trim().split(/\s+/g).forEach((type) => {
  target.removeEventListener(type, handler, false);
});
function getWindowForElement(element) {
  const { defaultView, parentWindow } = element.ownerDocument || element;
  return defaultView || parentWindow || typeof window !== "undefined" && window;
}
var Input = class {
  constructor(manager) {
    this.manager = manager;
    this.callback = inputHandler;
    this.element = manager.element;
    this.target = manager.options.inputTarget;
    this.domHandler = (ev) => {
      if (manager.options.enable) this.handler(ev);
    };
  }
  /**
   * @private
   * should handle the inputEvent data and trigger the callback
   * @virtual
   */
  handler() {
  }
  /**
   * @private
   * bind the events
   */
  init() {
    const { element, evEl, evTarget, evWin, domHandler, target } = this;
    if (evEl) addEventListeners(element, evEl, domHandler);
    if (evTarget) addEventListeners(target, evTarget, domHandler);
    if (evWin) {
      addEventListeners(getWindowForElement(element), evWin, domHandler);
    }
  }
  /**
   * @private
   * unbind the events
   */
  destroy() {
    const { element, evEl, evTarget, evWin, domHandler, target } = this;
    if (evEl) removeEventListeners(element, evEl, domHandler);
    if (evTarget) removeEventListeners(target, evTarget, domHandler);
    if (evWin) {
      removeEventListeners(
        getWindowForElement(element),
        evWin,
        domHandler
      );
    }
  }
};

// lib/verso-browser/vendor/hammer/input/mouse.js
var MOUSE_INPUT_MAP = {
  mousedown: INPUT_START,
  mousemove: INPUT_MOVE,
  mouseup: INPUT_END
};
var MOUSE_ELEMENT_EVENTS = "mousedown";
var MOUSE_WINDOW_EVENTS = "mousemove mouseup";
var MouseInput = class extends Input {
  // mousedown state
  constructor() {
    super(...arguments);
    __publicField(this, "evEl", MOUSE_ELEMENT_EVENTS);
    __publicField(this, "evWin", MOUSE_WINDOW_EVENTS);
    __publicField(this, "pressed", false);
    this.init();
  }
  /**
   * @private
   * handle mouse events
   * @param {Object} ev
   */
  handler(ev) {
    let eventType = MOUSE_INPUT_MAP[ev.type];
    if (eventType & INPUT_START && ev.button === 0) this.pressed = true;
    if (eventType & INPUT_MOVE && ev.which !== 1) eventType = INPUT_END;
    if (!this.pressed) return;
    if (eventType & INPUT_END) this.pressed = false;
    this.callback(this.manager, eventType, {
      pointers: [ev],
      changedPointers: [ev],
      pointerType: INPUT_TYPE_MOUSE,
      srcEvent: ev
    });
  }
};

// lib/verso-browser/vendor/hammer/input/pointerevent.js
var POINTER_INPUT_MAP = {
  pointerdown: INPUT_START,
  pointermove: INPUT_MOVE,
  pointerup: INPUT_END,
  pointercancel: INPUT_CANCEL,
  pointerout: INPUT_CANCEL
};
var POINTER_ELEMENT_EVENTS = "pointerdown";
var POINTER_WINDOW_EVENTS = "pointermove pointerup pointercancel";
var PointerEventInput = class extends Input {
  constructor() {
    super(...arguments);
    __publicField(this, "evEl", POINTER_ELEMENT_EVENTS);
    __publicField(this, "evWin", POINTER_WINDOW_EVENTS);
    this.store = this.manager.session.pointerEvents = [];
    this.init();
  }
  /**
   * @private
   * handle mouse events
   * @param {Object} ev
   */
  handler(ev) {
    const { store } = this;
    let removePointer = false;
    const eventTypeNormalized = ev.type.toLowerCase().replace("ms", "");
    const eventType = POINTER_INPUT_MAP[eventTypeNormalized];
    const pointerType = ev.pointerType;
    const isTouch = pointerType === INPUT_TYPE_TOUCH;
    let storeIndex = store.findIndex(
      ({ pointerId }) => pointerId === ev.pointerId
    );
    if (eventType & INPUT_START && (ev.button === 0 || isTouch)) {
      if (storeIndex < 0) {
        store.push(ev);
        storeIndex = store.length - 1;
      }
    } else if (eventType & (INPUT_END | INPUT_CANCEL)) {
      removePointer = true;
    }
    if (storeIndex < 0) return;
    store[storeIndex] = ev;
    this.callback(this.manager, eventType, {
      pointers: store,
      changedPointers: [ev],
      pointerType,
      srcEvent: ev
    });
    if (removePointer) store.splice(storeIndex, 1);
  }
};

// lib/verso-browser/vendor/hammer/input/touch.js
var TouchInput = class extends Input {
  constructor() {
    super(...arguments);
    __publicField(this, "evTarget", "touchstart touchmove touchend touchcancel");
    __publicField(this, "targetIds", {});
    this.init();
  }
  handler(ev) {
    const type = TOUCH_INPUT_MAP[ev.type];
    const touches = this.getTouches(ev, type);
    if (!touches) return;
    this.callback(this.manager, type, {
      pointers: touches[0],
      changedPointers: touches[1],
      pointerType: INPUT_TYPE_TOUCH,
      srcEvent: ev
    });
  }
  getTouches(ev, type) {
    const allTouches = Array.from(ev.touches);
    const { targetIds } = this;
    if (type & (INPUT_START | INPUT_MOVE) && allTouches.length === 1) {
      targetIds[allTouches[0].identifier] = true;
      return [allTouches, allTouches];
    }
    const targetTouches = allTouches.filter(
      (touch) => this.target.contains(touch.target)
    );
    if (type === INPUT_START) {
      targetTouches.forEach((targetTouch) => {
        targetIds[targetTouch.identifier] = true;
      });
    }
    const changedTargetTouches = [];
    Array.from(ev.changedTouches).forEach((changedTouch) => {
      if (targetIds[changedTouch.identifier]) {
        changedTargetTouches.push(changedTouch);
      }
      if (type & (INPUT_END | INPUT_CANCEL)) {
        delete targetIds[changedTouch.identifier];
      }
    });
    if (!changedTargetTouches.length) return;
    return [
      targetTouches.concat(changedTargetTouches).filter(
        (item, i, list) => list.findIndex(
          ({ identifier }) => identifier === item.identifier
        ) === i
      ),
      changedTargetTouches
    ];
  }
};

// lib/verso-browser/vendor/hammer/input/touchmouse.js
var DEDUP_TIMEOUT = 2500;
var DEDUP_DISTANCE = 25;
var TouchMouseInput = class extends Input {
  constructor() {
    super(...arguments);
    __publicField(this, "primaryTouch", null);
    __publicField(this, "lastTouches", []);
    /**
     * @private
     * handle mouse and touch events
     * @param {Hammer} manager
     * @param {String} inputEvent
     * @param {Object} inputData
     */
    __publicField(this, "handler", (manager, inputEvent, inputData) => {
      const isTouch = inputData.pointerType === INPUT_TYPE_TOUCH;
      const isMouse = inputData.pointerType === INPUT_TYPE_MOUSE;
      if (isMouse && inputData.sourceCapabilities && inputData.sourceCapabilities.firesTouchEvents) {
        return;
      }
      if (isTouch) {
        if (inputEvent & INPUT_START) {
          this.primaryTouch = inputData.changedPointers[0].identifier;
          this.setLastTouch(inputData);
        } else if (inputEvent & (INPUT_END | INPUT_CANCEL)) {
          this.setLastTouch(inputData);
        }
      } else if (isMouse && this.isSyntheticEvent(inputData)) {
        return;
      }
      this.callback(manager, inputEvent, inputData);
    });
    this.touch = new TouchInput(this.manager, this.handler);
    this.mouse = new MouseInput(this.manager, this.handler);
    this.init();
  }
  /**
   * @private
   * remove the event listeners
   */
  destroy() {
    this.touch.destroy();
    this.mouse.destroy();
  }
  isSyntheticEvent({ srcEvent: { clientX, clientY } }) {
    return this.lastTouches.some(
      ({ x, y }) => Math.abs(clientX - x) <= DEDUP_DISTANCE && Math.abs(clientY - y) <= DEDUP_DISTANCE
    );
  }
  setLastTouch({ changedPointers: [{ identifier, clientX, clientY }] }) {
    if (identifier === this.primaryTouch) {
      const lastTouch = { x: clientX, y: clientY };
      this.lastTouches.push(lastTouch);
      const lts = this.lastTouches;
      setTimeout(() => {
        const i = lts.indexOf(lastTouch);
        if (i > -1) lts.splice(i, 1);
      }, DEDUP_TIMEOUT);
    }
  }
};

// lib/verso-browser/vendor/hammer/Recognizer.js
var STATE_POSSIBLE = 1;
var STATE_BEGAN = 2;
var STATE_CHANGED = 4;
var STATE_ENDED = 8;
var STATE_RECOGNIZED = STATE_ENDED;
var STATE_CANCELLED = 16;
var STATE_FAILED = 32;
function stateStr(state) {
  if (state & STATE_CANCELLED) return "cancel";
  if (state & STATE_ENDED) return "end";
  if (state & STATE_CHANGED) return "move";
  if (state & STATE_BEGAN) return "start";
  return "";
}
var _uniqueId = 1;
var uniqueId = () => _uniqueId++;
var Recognizer = class {
  constructor(options) {
    __publicField(this, "id", uniqueId());
    __publicField(this, "manager", null);
    __publicField(this, "state", STATE_POSSIBLE);
    __publicField(this, "simultaneous", {});
    __publicField(this, "requireFail", []);
    var _a2;
    this.options = __spreadValues(__spreadValues({}, this.defaults), options);
    this.options.enable = (_a2 = this.options.enable) != null ? _a2 : true;
  }
  /**
   * @private
   * set options
   * @param {Object} options
   * @return {Recognizer}
   */
  set(options) {
    Object.assign(this.options, options);
    if (this.manager) this.manager.touchAction.update();
    return this;
  }
  /**
   * @private
   * You should use `tryEmit` instead of `emit` directly to check
   * that all the needed recognizers has failed before emitting.
   * @param {Object} input
   */
  emit(input) {
    const { manager, options, state } = this;
    const emit = (event) => manager.emit(event, input);
    if (state < STATE_ENDED) emit(options.event + stateStr(state));
    emit(this.options.event);
    if (input.additionalEvent) emit(input.additionalEvent);
    if (state >= STATE_ENDED) emit(options.event + stateStr(state));
  }
  /**
   * @private
   * update the recognizer
   * @param {Object} inputData
   */
  recognize(inputData) {
    if (!this.options.enable) {
      this.reset();
      this.state = STATE_FAILED;
      return;
    }
    if (this.state & (STATE_RECOGNIZED | STATE_CANCELLED | STATE_FAILED)) {
      this.state = STATE_POSSIBLE;
    }
    const inputDataClone = __spreadValues({}, inputData);
    this.state = this.process(inputDataClone);
    if (this.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED | STATE_CANCELLED)) {
      this.emit(inputDataClone);
    }
  }
  /**
   * @private
   * return the state of the recognizer
   * the actual recognizing happens in this method
   * @virtual
   * @param {Object} inputData
   * @returns {constant} STATE
   */
  /* jshint ignore:start */
  process() {
  }
  /* jshint ignore:end */
  /**
   * @private
   * return the preferred touch-action
   * @virtual
   * @returns {Array}
   */
  getTouchAction() {
  }
  /**
   * @private
   * called when the gesture isn't allowed to recognize
   * like when another is being recognized or it is disabled
   * @virtual
   */
  reset() {
  }
};

// lib/verso-browser/vendor/hammer/utils/prefixed.ts
var VENDOR_PREFIXES = ["", "webkit", "Moz", "MS", "ms", "o"];
function prefixed(obj, property) {
  const camelProp = property[0].toUpperCase() + property.slice(1);
  return VENDOR_PREFIXES.find(
    (prefix) => (prefix ? prefix + camelProp : property) in obj
  );
}

// lib/verso-browser/vendor/hammer/TouchAction.ts
var PREFIXED_TOUCH_ACTION = typeof document !== "undefined" && prefixed(document.createElement("div").style, "touchAction");
var TOUCH_ACTION_COMPUTE = "compute";
var TOUCH_ACTION_AUTO = "auto";
var TOUCH_ACTION_MANIPULATION = "manipulation";
var TOUCH_ACTION_PAN_X = "pan-x";
var TOUCH_ACTION_PAN_Y = "pan-y";
var TOUCH_ACTION_NONE = "none";
var actions = [
  "auto",
  "manipulation",
  "pan-y",
  "pan-x",
  "pan-x pan-y",
  "none"
];
var _a;
var cssSupports = typeof window !== "undefined" && ((_a = window.CSS) == null ? void 0 : _a.supports);
var TOUCH_ACTION_MAP = PREFIXED_TOUCH_ACTION && actions.reduce((touchMap, val) => {
  touchMap[val] = cssSupports ? cssSupports("touch-action", val) : true;
  return touchMap;
}, {});
function cleanTouchActions(actionsToClean) {
  if (actionsToClean.includes(TOUCH_ACTION_NONE)) return TOUCH_ACTION_NONE;
  const hasPanX = actionsToClean.includes(TOUCH_ACTION_PAN_X);
  const hasPanY = actionsToClean.includes(TOUCH_ACTION_PAN_Y);
  if (hasPanX && hasPanY) return TOUCH_ACTION_NONE;
  if (hasPanX) return TOUCH_ACTION_PAN_X;
  if (hasPanY) return TOUCH_ACTION_PAN_Y;
  const hasManipulation = actionsToClean.includes(TOUCH_ACTION_MANIPULATION);
  if (hasManipulation) return TOUCH_ACTION_MANIPULATION;
  return TOUCH_ACTION_AUTO;
}
var TouchAction = class {
  constructor(manager, value) {
    __publicField(this, "manager");
    __publicField(this, "actions");
    this.manager = manager;
    this.set(value);
  }
  /**
   * set the touchAction value on the element or enable the polyfill
   */
  set(value) {
    if (value === TOUCH_ACTION_COMPUTE) value = this.compute();
    if (PREFIXED_TOUCH_ACTION && this.manager.element.style && (TOUCH_ACTION_MAP == null ? void 0 : TOUCH_ACTION_MAP[value])) {
      this.manager.element.style[PREFIXED_TOUCH_ACTION] = value;
    }
    this.actions = value.toLowerCase().trim();
  }
  /**
   * just re-set the touchAction value
   */
  update() {
    this.set(this.manager.options.touchAction);
  }
  /**
   * @private
   * compute the value for the touchAction property based on the recognizer's settings
   */
  compute() {
    return cleanTouchActions(
      this.manager.recognizers.reduce(
        (actions2, recognizer) => recognizer.options.enable ? actions2.concat(recognizer.getTouchAction()) : actions2,
        []
      ).join(" ")
    );
  }
  /**
   * this method is called on each input cycle and provides the preventing of the browser behavior
   */
  preventDefaults({
    srcEvent,
    pointers,
    distance,
    deltaTime,
    offsetDirection
  }) {
    if (this.manager.session.prevented) return srcEvent.preventDefault();
    const hasNone = this.actions.includes(TOUCH_ACTION_NONE) && !(TOUCH_ACTION_MAP == null ? void 0 : TOUCH_ACTION_MAP[TOUCH_ACTION_NONE]);
    if (hasNone && pointers.length === 1 && distance < 2 && deltaTime < 250)
      return;
    const hasPanY = this.actions.includes(TOUCH_ACTION_PAN_Y) && !(TOUCH_ACTION_MAP == null ? void 0 : TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_Y]);
    const hasPanX = this.actions.includes(TOUCH_ACTION_PAN_X) && !(TOUCH_ACTION_MAP == null ? void 0 : TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_X]);
    if (hasPanX && hasPanY) return;
    if (hasNone || hasPanY && offsetDirection & DIRECTION_HORIZONTAL || hasPanX && offsetDirection & DIRECTION_VERTICAL) {
      this.manager.session.prevented = true;
      srcEvent.preventDefault();
    }
  }
};

// lib/verso-browser/vendor/hammer/Manager.js
var SUPPORT_TOUCH = typeof window !== "undefined" && "ontouchstart" in window;
var SUPPORT_ONLY_TOUCH = SUPPORT_TOUCH && /mobile|tablet|ip(ad|hone|od)|android/i.test(navigator.userAgent);
var SUPPORT_POINTER_EVENTS = typeof window !== "undefined" && Boolean(prefixed(window, "PointerEvent"));
function createInputInstance(manager) {
  const { inputClass } = manager.options;
  if (inputClass) return new inputClass(manager);
  if (SUPPORT_POINTER_EVENTS) return new PointerEventInput(manager);
  if (SUPPORT_ONLY_TOUCH) return new TouchInput(manager);
  if (!SUPPORT_TOUCH) return new MouseInput(manager);
  return new TouchMouseInput(manager);
}
var defaults = {
  /**
   * @private
   * set if DOM events are being triggered.
   * But this is slower and unused by simple implementations, so disabled by default.
   * @type {Boolean}
   * @default false
   */
  domEvents: false,
  /**
   * @private
   * The value for the touchAction property/fallback.
   * When set to `compute` it will magically set the correct value based on the added recognizers.
   * @type {String}
   * @default compute
   */
  touchAction: TOUCH_ACTION_COMPUTE,
  /**
   * @private
   * @type {Boolean}
   * @default true
   */
  enable: true,
  /**
   * @private
   * EXPERIMENTAL FEATURE -- can be removed/changed
   * Change the parent input target element.
   * If Null, then it is being set the to main element.
   * @type {Null|EventTarget}
   * @default null
   */
  inputTarget: null,
  /**
   * @private
   * force an input class
   * @type {Null|Function}
   * @default null
   */
  inputClass: null,
  /**
   * @private
   * Some CSS properties can be used to improve the working of Hammer.
   * Add them to this method and they will be set when creating a new Manager.
   * @namespace
   */
  cssProps: {
    /**
     * @private
     * Disables text selection to improve the dragging gesture. Mainly for desktop browsers.
     * @type {String}
     * @default 'none'
     */
    userSelect: "none",
    /**
     * @private
     * Disable the Windows Phone grippers when pressing an element.
     * @type {String}
     * @default 'none'
     */
    touchSelect: "none",
    /**
     * @private
     * Disables the default callout shown when you touch and hold a touch target.
     * On iOS, when you touch and hold a touch target such as a link, Safari displays
     * a callout containing information about the link. This property allows you to disable that callout.
     * @type {String}
     * @default 'none'
     */
    touchCallout: "none",
    /**
     * @private
     * Specifies that an entire element should be draggable instead of its contents. Mainly for desktop browsers.
     * @type {String}
     * @default 'none'
     */
    userDrag: "none",
    /**
     * @private
     * Overrides the highlight color shown when the user taps a link or a JavaScript
     * clickable element in iOS. This property obeys the alpha value, if specified.
     * @type {String}
     * @default 'rgba(0,0,0,0)'
     */
    tapHighlightColor: "rgba(0,0,0,0)"
  }
};
var Manager = class {
  constructor(element, options) {
    __publicField(this, "handlers", {});
    __publicField(this, "session", {});
    __publicField(this, "recognizers", []);
    __publicField(this, "oldCssProps", {});
    this.options = __spreadValues(__spreadValues({}, defaults), options);
    this.options.inputTarget = this.options.inputTarget || element;
    this.element = element;
    this.input = createInputInstance(this);
    this.touchAction = new TouchAction(this, this.options.touchAction);
    this.toggleCssProps(true);
    this.options.recognizers.forEach(({ 0: ctor, 1: opts }) => {
      this.add(new ctor(opts));
    }, this);
  }
  /**
   * set options
   * @param {Object} options
   * @returns {Manager}
   */
  set(options) {
    Object.assign(this.options, options);
    if (options.touchAction) this.touchAction.update();
    if (options.inputClass) this.input = createInputInstance(this);
    if (options.inputTarget) {
      this.input.destroy();
      this.input.target = options.inputTarget;
      this.input.init();
    }
    return this;
  }
  /**
   * @private
   * run the recognizers!
   * called by the inputHandler function on every movement of the pointers (touches)
   * it walks through all the recognizers and tries to detect the gesture that is being made
   * @param {Object} inputData
   */
  recognize(inputData) {
    const { session } = this;
    this.touchAction.preventDefaults(inputData);
    let { curRecognizer } = session;
    if (!curRecognizer || curRecognizer && curRecognizer.state & STATE_RECOGNIZED) {
      curRecognizer = session.curRecognizer = null;
    }
    this.recognizers.forEach((recognizer) => {
      if (!curRecognizer || recognizer === curRecognizer) {
        recognizer.recognize(inputData);
      } else {
        recognizer.reset();
      }
      if (!curRecognizer && recognizer.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED)) {
        curRecognizer = session.curRecognizer = recognizer;
      }
    });
  }
  /**
   * get a recognizer by its event name.
   * @param {Recognizer|String} recognizer
   * @returns {Recognizer|undefined}
   */
  get(recognizer) {
    return recognizer instanceof Recognizer ? recognizer : this.recognizers.find(
      (recogger) => recogger.options.event === recognizer
    );
  }
  /**
   * existing recognizers with the same event name will be removed
   * @param {Recognizer} recognizer
   * @returns {Recognizer|Manager}
   */
  add(recognizer) {
    const existing = this.get(recognizer.options.event);
    if (existing) this.remove(existing);
    this.recognizers.push(recognizer);
    recognizer.manager = this;
    this.touchAction.update();
    return recognizer;
  }
  /**
   * remove a recognizer by name or instance
   * @param {Recognizer|String} recognizer
   * @returns {Manager}
   */
  remove(recognizer) {
    recognizer = this.get(recognizer);
    if (recognizer) {
      const index = this.recognizers.indexOf(recognizer);
      if (index !== -1) {
        this.recognizers.splice(index, 1);
        this.touchAction.update();
      }
    }
    return this;
  }
  /**
   * bind event
   * @param {String} events
   * @param {Function} handler
   * @returns {EventEmitter} this
   */
  on(events, handler) {
    if (events === void 0 || handler === void 0) return;
    const { handlers } = this;
    events.trim().split(/\s+/g).forEach((event) => {
      handlers[event] = handlers[event] || [];
      handlers[event].push(handler);
    });
    return this;
  }
  /**
   * unbind event, leave emit blank to remove all handlers
   * @param {String} events
   * @param {Function} [handler]
   * @returns {EventEmitter} this
   */
  off(events, handler) {
    if (events === void 0) return;
    events.trim().split(/\s+/g).forEach((event) => {
      if (!handler) {
        delete this.handlers[event];
      } else if (this.handlers[event]) {
        this.handlers[event].splice(
          this.handlers[event].indexOf(handler),
          1
        );
      }
    });
    return this;
  }
  /**
   * @private emit event to the listeners
   * @param {String} event
   * @param {Object} data
   */
  emit(event, data) {
    if (this.options.domEvents) triggerDomEvent(event, data);
    const handlers = this.handlers[event] && this.handlers[event].slice();
    if (!handlers || !handlers.length) return;
    data.type = event;
    data.preventDefault = () => {
      data.srcEvent.preventDefault();
    };
    handlers.forEach((handler) => {
      handler(data);
    });
  }
  /**
   * destroy the manager and unbinds all events
   * it doesn't unbind dom events, that is the user own responsibility
   */
  destroy() {
    if (this.element) this.toggleCssProps(false);
    this.handlers = {};
    this.session = {};
    this.input.destroy();
    this.element = null;
  }
  toggleCssProps(add) {
    const { element } = this;
    if (!element.style) return;
    Object.entries(this.options.cssProps).forEach(({ 0: value, 1: name }) => {
      const prop = prefixed(element.style, name);
      if (add) {
        this.oldCssProps[prop] = element.style[prop];
        element.style[prop] = value;
      } else {
        element.style[prop] = this.oldCssProps[prop] || "";
      }
    });
    if (!add) this.oldCssProps = {};
  }
};
function triggerDomEvent(event, data) {
  const gestureEvent = document.createEvent("Event");
  gestureEvent.initEvent(event, true, true);
  gestureEvent.gesture = data;
  data.target.dispatchEvent(gestureEvent);
}

// lib/verso-browser/vendor/hammer/recognizers/attribute.js
var AttrRecognizer = class extends Recognizer {
  /**
   * @private
   * Used to check if it the recognizer receives valid input, like input.distance > 10.
   * @memberof AttrRecognizer
   * @param {Object} input
   * @returns {Boolean} recognized
   */
  attrTest(input) {
    const optionPointers = this.options.pointers;
    return optionPointers === 0 || input.pointers.length === optionPointers;
  }
  /**
   * @private
   * Process the input and return the state for the recognizer
   * @memberof AttrRecognizer
   * @param {Object} input
   * @returns {*} State
   */
  process(input) {
    const { state } = this;
    const isRecognized = state & (STATE_BEGAN | STATE_CHANGED);
    const isValid = this.attrTest(input);
    if (isRecognized && (input.eventType & INPUT_CANCEL || !isValid)) {
      return state | STATE_CANCELLED;
    }
    if (isRecognized || isValid) {
      if (input.eventType & INPUT_END) return state | STATE_ENDED;
      if (!(state & STATE_BEGAN)) return STATE_BEGAN;
      return state | STATE_CHANGED;
    }
    return STATE_FAILED;
  }
};
AttrRecognizer.prototype.defaults = {
  /**
   * @private
   * @type {Number}
   * @default 1
   */
  pointers: 1
};

// lib/verso-browser/vendor/hammer/recognizers/pan.js
var PanRecognizer = class extends AttrRecognizer {
  constructor() {
    super(...arguments);
    __publicField(this, "pX", null);
    __publicField(this, "pY", null);
  }
  getTouchAction() {
    const {
      options: { direction }
    } = this;
    const actions2 = [];
    if (direction & DIRECTION_HORIZONTAL) actions2.push(TOUCH_ACTION_PAN_Y);
    if (direction & DIRECTION_VERTICAL) actions2.push(TOUCH_ACTION_PAN_X);
    return actions2;
  }
  directionTest(input) {
    const { options } = this;
    const { deltaX, deltaY } = input;
    let { distance } = input;
    let hasMoved = true;
    if (!(input.direction & options.direction)) {
      if (options.direction & DIRECTION_HORIZONTAL) {
        input.direction = deltaX === 0 ? DIRECTION_NONE : deltaX < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
        hasMoved = deltaX !== this.pX;
        distance = Math.abs(deltaX);
      } else {
        input.direction = deltaY === 0 ? DIRECTION_NONE : deltaY < 0 ? DIRECTION_UP : DIRECTION_DOWN;
        hasMoved = deltaY !== this.pY;
        distance = Math.abs(deltaY);
      }
    }
    return hasMoved && distance > options.threshold && input.direction & options.direction;
  }
  attrTest(input) {
    return super.attrTest(input) && (this.state & STATE_BEGAN || !(this.state & STATE_BEGAN) && this.directionTest(input));
  }
  emit(input) {
    this.pX = input.deltaX;
    this.pY = input.deltaY;
    const direction = directionStr(input.direction);
    if (direction) input.additionalEvent = this.options.event + direction;
    super.emit(input);
  }
};
PanRecognizer.prototype.defaults = {
  event: "pan",
  threshold: 10,
  pointers: 1,
  direction: DIRECTION_ALL
};

// lib/verso-browser/vendor/hammer/recognizers/pinch.js
var PinchRecognizer = class extends AttrRecognizer {
  getTouchAction() {
    return [TOUCH_ACTION_NONE];
  }
  attrTest(input) {
    return super.attrTest(input) && (Math.abs(input.scale - 1) > this.options.threshold || this.state & STATE_BEGAN);
  }
  emit(input) {
    if (input.scale !== 1) {
      const inOut = input.scale < 1 ? "in" : "out";
      input.additionalEvent = this.options.event + inOut;
    }
    super.emit(input);
  }
};
PinchRecognizer.prototype.defaults = {
  event: "pinch",
  threshold: 0,
  pointers: 2
};

// lib/verso-browser/vendor/hammer/recognizers/press.js
var PressRecognizer = class extends Recognizer {
  constructor() {
    super(...arguments);
    __publicField(this, "_timer", null);
    __publicField(this, "_input", null);
  }
  getTouchAction() {
    return [TOUCH_ACTION_AUTO];
  }
  process(input) {
    const { options } = this;
    const validPointers = input.pointers.length === options.pointers;
    const validMovement = input.distance < options.threshold;
    const validTime = input.deltaTime > options.time;
    this._input = input;
    if (!validMovement || !validPointers || input.eventType & (INPUT_END | INPUT_CANCEL) && !validTime) {
      this.reset();
    } else if (input.eventType & INPUT_START) {
      this.reset();
      this._timer = setTimeout(() => {
        this.state = STATE_RECOGNIZED;
        this.tryEmit();
      }, options.time);
    } else if (input.eventType & INPUT_END) {
      return STATE_RECOGNIZED;
    }
    return STATE_FAILED;
  }
  reset() {
    clearTimeout(this._timer);
  }
  emit(input) {
    if (this.state !== STATE_RECOGNIZED) return;
    if (input && input.eventType & INPUT_END) {
      this.manager.emit("".concat(this.options.event, "up"), input);
    } else {
      this._input.timeStamp = Date.now();
      this.manager.emit(this.options.event, this._input);
    }
  }
  /**
   * @private
   * Check that all the require failure recognizers has failed,
   * if true, it emits a gesture event,
   * otherwise, setup the state to FAILED.
   * @param {Object} input
   */
  tryEmit(input) {
    if (this.canEmit()) return this.emit(input);
    this.state = STATE_FAILED;
  }
  /**
   * @private
   * can we emit?
   * @returns {boolean}
   */
  canEmit() {
    let i = 0;
    while (i < this.requireFail.length) {
      if (!(this.requireFail[i].state & (STATE_FAILED | STATE_POSSIBLE)))
        return false;
      i++;
    }
    return true;
  }
};
PressRecognizer.prototype.defaults = {
  event: "press",
  pointers: 1,
  time: 251,
  // minimal time of the pointer to be pressed
  threshold: 9
  // a minimal movement is ok, but keep it low
};

// lib/verso-browser/vendor/hammer/recognizers/tap.js
var TapRecognizer = class extends Recognizer {
  constructor() {
    super(...arguments);
    // previous time and center,
    // used for tap counting
    __publicField(this, "pTime", false);
    __publicField(this, "pCenter", false);
    __publicField(this, "_timer", null);
    __publicField(this, "_input", null);
    __publicField(this, "count", 0);
  }
  getTouchAction() {
    return [TOUCH_ACTION_MANIPULATION];
  }
  process(input) {
    const { options } = this;
    const validPointers = input.pointers.length === options.pointers;
    const validMovement = input.distance < options.threshold;
    const validTouchTime = input.deltaTime < options.time;
    this.reset();
    if (input.eventType & INPUT_START && this.count === 0) {
      return this.failTimeout();
    }
    if (validMovement && validTouchTime && validPointers) {
      if (input.eventType !== INPUT_END) return this.failTimeout();
      const validInterval = this.pTime ? input.timeStamp - this.pTime < options.interval : true;
      const validMultiTap = !this.pCenter || getDistance(this.pCenter, input.center) < options.posThreshold;
      this.pTime = input.timeStamp;
      this.pCenter = input.center;
      this.count = !validMultiTap || !validInterval ? 1 : this.count + 1;
      this._input = input;
      if (this.count % options.taps === 0) return STATE_RECOGNIZED;
    }
    return STATE_FAILED;
  }
  failTimeout() {
    this._timer = setTimeout(() => {
      this.state = STATE_FAILED;
    }, this.options.interval);
    return STATE_FAILED;
  }
  reset() {
    clearTimeout(this._timer);
  }
  emit() {
    if (this.state === STATE_RECOGNIZED) {
      this._input.tapCount = this.count;
      this.manager.emit(this.options.event, this._input);
    }
  }
};
TapRecognizer.prototype.defaults = {
  event: "tap",
  pointers: 1,
  taps: 1,
  interval: 300,
  // max time between the multi-tap taps
  time: 250,
  // max time of the pointer to be down (like finger on the screen)
  threshold: 9,
  // a minimal movement is ok, but keep it low
  posThreshold: 10
  // a multi-tap can be a bit off the initial position
};

// lib/verso-browser/verso.ts
var buildPageIds = (pageSpreads) => pageSpreads.reduce(
  (pageIds, pageSpread) => {
    pageSpread.options.pageIds.forEach((pageId) => {
      pageIds[pageId] = pageSpread;
    });
    return pageIds;
  },
  {}
);
var clipCoordinate = (coordinate, scale, size, offset) => size * scale < 100 ? offset * -scale + 50 - size * scale / 2 : Math.max(
  Math.min(coordinate, offset * -scale),
  offset * -scale - size * scale + 100
);
function getPageSpreadBounds(pageSpread) {
  const pageSpreadRect = pageSpread.getRect();
  const pageSpreadContentRect = pageSpread.getContentRect();
  return {
    left: (pageSpreadContentRect.left - pageSpreadRect.left) / pageSpreadRect.width * 100,
    top: (pageSpreadContentRect.top - pageSpreadRect.top) / pageSpreadRect.height * 100,
    width: pageSpreadContentRect.width / pageSpreadRect.width * 100,
    height: pageSpreadContentRect.height / pageSpreadRect.height * 100,
    pageSpreadRect,
    pageSpreadContentRect
  };
}
function isCoordinateInsideElement(x, y, el) {
  const { left, right, top, bottom } = el.getBoundingClientRect();
  return x >= left && x <= right && y >= top && y <= bottom;
}
function traversePageSpreads(els) {
  var _a2, _b, _c;
  const pageSpreads = [];
  let left = 0;
  for (let i = 0; i < els.length; i++) {
    const el = els[i];
    const width = Number((_a2 = el.dataset.width) != null ? _a2 : 100);
    const pageSpread = new PageSpread(el, {
      // @ts-expect-error - io: I don't understand how a lot of this works downstream when data-id is missing, but it does
      id: el.dataset.id,
      type: el.dataset.type,
      pageIds: ((_b = el.dataset.pageIds) == null ? void 0 : _b.split(",")) || [],
      maxZoomScale: Number((_c = el.dataset.maxZoomScale) != null ? _c : 1),
      width,
      left
    });
    left += width;
    pageSpreads.push(pageSpread);
  }
  return pageSpreads;
}
var Verso = class extends microevent_default {
  constructor(el, options = {}) {
    var _a2, _b, _c, _d, _e, _f;
    super();
    __publicField(this, "position", -1);
    __publicField(this, "pinching", false);
    __publicField(this, "panning", false);
    __publicField(this, "transform", { left: 0, top: 0, scale: 1 });
    __publicField(this, "startTransform", { left: 0, top: 0, scale: 1 });
    __publicField(this, "started", false);
    __publicField(this, "destroyed", false);
    __publicField(this, "_events", {});
    __publicField(this, "el");
    __publicField(this, "scrollerEl");
    __publicField(this, "pageSpreadEls");
    __publicField(this, "pageSpreads");
    __publicField(this, "pageIds");
    __publicField(this, "options");
    __publicField(this, "swipeVelocity");
    __publicField(this, "swipeThreshold");
    __publicField(this, "navigationDuration");
    __publicField(this, "navigationPanDuration");
    __publicField(this, "zoomDuration");
    __publicField(this, "tap");
    __publicField(this, "animation");
    __publicField(this, "hammer");
    __publicField(this, "coarsePointerQuery");
    __publicField(this, "prefersReducedMotionQuery");
    //#############
    /* Events */
    //#############
    __publicField(this, "onPanStart", (e) => {
      if (this.transform.scale > 1 || e.direction === DIRECTION_LEFT || e.direction === DIRECTION_RIGHT) {
        const { x } = e.center;
        const edgeThreshold = 30;
        const width = this.scrollerEl.offsetWidth;
        if (x > edgeThreshold && x < width - edgeThreshold) {
          this.startTransform.left = this.transform.left;
          this.startTransform.top = this.transform.top;
          this.panning = true;
          this.trigger("panStart");
        }
      }
    });
    __publicField(this, "onPanMove", (e) => {
      if (this.pinching || !this.panning) return;
      const { scale } = this.transform;
      if (scale > 1) {
        const activePageSpread = this.getActivePageSpread();
        const carouselOffset = activePageSpread.getLeft();
        const carouselScaledOffset = carouselOffset * scale;
        const { width, height, left, top } = getPageSpreadBounds(activePageSpread);
        let x = this.startTransform.left + carouselScaledOffset + e.deltaX / this.scrollerEl.offsetWidth * 100;
        x = clipCoordinate(x, scale, width, left) - carouselScaledOffset;
        let y = this.startTransform.top + e.deltaY / this.scrollerEl.offsetHeight * 100;
        y = clipCoordinate(y, scale, height, top);
        this.transform.left = x;
        this.transform.top = y;
        this.animation.animate({
          x: x + "%",
          y: y + "%",
          scale,
          easing: "linear"
        });
      } else {
        this.animation.animate({
          x: this.transform.left + e.deltaX / this.scrollerEl.offsetWidth * 100 + "%",
          easing: "linear"
        });
      }
    });
    __publicField(this, "onPanEnd", (e) => {
      if (!this.panning) return;
      this.panning = false;
      this.trigger("panEnd");
      if (this.transform.scale === 1 && !this.pinching) {
        const position = this.getPosition();
        const velocity = e.overallVelocityX;
        if (Math.abs(velocity) >= this.swipeVelocity && Math.abs(e.deltaX) >= this.swipeThreshold) {
          const options = {
            velocity,
            duration: this.navigationPanDuration
          };
          if (e.offsetDirection === DIRECTION_LEFT) {
            this.next(options);
          } else if (e.offsetDirection === DIRECTION_RIGHT) {
            this.prev(options);
          }
        }
        if (position === this.getPosition()) {
          this.animation.animate({
            x: this.transform.left + "%",
            duration: this.navigationPanDuration
          });
          this.trigger("attemptedNavigation", {
            position: this.getPosition()
          });
        }
      }
    });
    __publicField(this, "onPinchStart", () => {
      if (!this.getActivePageSpread().isZoomable()) return;
      this.pinching = true;
      this.el.dataset.pinching = true;
      this.startTransform.scale = this.transform.scale;
    });
    __publicField(this, "onPinchMove", (e) => {
      if (!this.pinching) return;
      this.zoomTo({
        x: e.center.x,
        y: e.center.y,
        scale: this.startTransform.scale * e.scale,
        bounds: false,
        easing: "linear"
      });
    });
    __publicField(this, "onPinchEnd", (e) => {
      if (!this.pinching) return;
      const activePageSpread = this.getActivePageSpread();
      const maxZoomScale = activePageSpread.getMaxZoomScale();
      const scale = Math.max(1, Math.min(this.transform.scale, maxZoomScale));
      const position = this.getPosition();
      if (this.startTransform.scale === 1 && scale > 1) {
        this.trigger("zoomedIn", { position });
      } else if (this.startTransform.scale > 1 && scale === 1) {
        this.trigger("zoomedOut", { position });
      }
      this.zoomTo(
        { x: e.center.x, y: e.center.y, scale, duration: this.zoomDuration },
        () => {
          this.pinching = false;
          this.el.dataset.pinching = false;
        }
      );
    });
    __publicField(this, "onPress", (e) => {
      this.trigger(
        "pressed",
        this.getCoordinateInfo(
          e.center.x,
          e.center.y,
          this.getActivePageSpread()
        )
      );
    });
    __publicField(this, "onContextmenu", (e) => {
      e.preventDefault();
      this.trigger(
        "contextmenu",
        this.getCoordinateInfo(
          e.clientX,
          e.clientY,
          this.getActivePageSpread()
        )
      );
      return false;
    });
    __publicField(this, "onWheel", (e) => {
      const activePageSpread = this.getActivePageSpread();
      if (!activePageSpread.isZoomable()) return;
      let { deltaY } = e;
      if (e.webkitDirectionInvertedFromDevice)
        deltaY = -deltaY;
      const position = this.getPosition();
      if (deltaY > 0 && this.transform.scale === 1) {
        this.zoomTo(
          {
            x: e.clientX,
            y: e.clientY,
            scale: activePageSpread.getMaxZoomScale(),
            duration: this.zoomDuration
          },
          () => {
            this.trigger("zoomedIn", { position });
          }
        );
      } else if (deltaY < 0 && this.transform.scale > 1) {
        this.zoomTo(
          {
            x: e.clientX,
            y: e.clientY,
            scale: 1,
            duration: this.zoomDuration
          },
          () => {
            this.trigger("zoomedOut", { position });
          }
        );
      }
    });
    __publicField(this, "onSingletap", (e) => {
      const activePageSpread = this.getActivePageSpread();
      const coordinateInfo = this.getCoordinateInfo(
        e.center.x,
        e.center.y,
        activePageSpread
      );
      this.trigger("pointerdown", coordinateInfo);
      clearTimeout(this.tap.timeout);
      if (this.tap.count === 1) {
        this.tap.count = 0;
        this.trigger("doubleClicked", coordinateInfo);
        if (activePageSpread.isZoomable()) {
          const maxZoomScale = activePageSpread.getMaxZoomScale();
          const zoomedIn = this.transform.scale > 1;
          const scale = zoomedIn ? 1 : maxZoomScale;
          const zoomEvent = zoomedIn ? "zoomedOut" : "zoomedIn";
          const position = this.getPosition();
          this.zoomTo(
            {
              x: e.center.x,
              y: e.center.y,
              scale,
              duration: this.zoomDuration
            },
            () => {
              this.trigger(zoomEvent, { position });
            }
          );
        }
      } else {
        this.tap.count++;
        this.tap.timeout = setTimeout(() => {
          this.tap.count = 0;
          this.trigger("clicked", coordinateInfo);
        }, this.tap.delay);
      }
    });
    __publicField(this, "onTouchStart", (e) => {
      if (!this.getActivePageSpread().isScrollable()) e.preventDefault();
    });
    __publicField(this, "onTouchEnd", (e) => {
      if (!this.getActivePageSpread().isScrollable()) e.preventDefault();
    });
    __publicField(this, "onResize", () => {
      if (this.transform.scale > 1) {
        const position = this.getPosition();
        this.transform.left = this.getLeftTransformFromPageSpread(
          position,
          this.getActivePageSpread()
        );
        this.transform.top = 0;
        this.transform.scale = 1;
        this.zoomTo({
          x: this.transform.left,
          y: this.transform.top,
          scale: this.transform.scale,
          duration: 0
        });
        this.trigger("zoomedOut", { position });
      }
    });
    this.el = el;
    this.options = options;
    this.swipeVelocity = (_a2 = this.options.swipeVelocity) != null ? _a2 : 0.3;
    this.swipeThreshold = (_b = this.options.swipeThreshold) != null ? _b : 10;
    this.navigationDuration = (_c = this.options.navigationDuration) != null ? _c : 240;
    this.navigationPanDuration = (_d = this.options.navigationPanDuration) != null ? _d : 200;
    this.zoomDuration = (_e = this.options.zoomDuration) != null ? _e : 200;
    this.tap = {
      count: 0,
      delay: (_f = this.options.doubleTapDelay) != null ? _f : 300,
      timeout: void 0
    };
  }
  start() {
    var _a2;
    this.coarsePointerQuery = typeof window !== "undefined" && window.matchMedia ? window.matchMedia("(pointer: coarse)") : null;
    this.scrollerEl = this.el.querySelector(".verso__scroller");
    this.pageSpreadEls = this.el.querySelectorAll(".verso__page-spread");
    this.pageSpreads = traversePageSpreads(this.pageSpreadEls);
    this.pageIds = buildPageIds(this.pageSpreads);
    this.animation = new Animation(this.scrollerEl);
    this.hammer = new Manager(this.scrollerEl, {
      touchAction: "none",
      enable: false,
      inputClass: ((_a2 = this.coarsePointerQuery) == null ? void 0 : _a2.matches) ? TouchInput : null,
      recognizers: [
        [PanRecognizer, { threshold: 5, direction: DIRECTION_ALL }],
        [TapRecognizer, { event: "singletap", interval: 0 }],
        [PinchRecognizer],
        [PressRecognizer, { time: 500 }]
      ]
    });
    if (this.coarsePointerQuery) {
      const onMedia = ({ matches }) => {
        this.hammer.set({ inputClass: matches ? TouchInput : null });
      };
      if (this.coarsePointerQuery.addEventListener) {
        this.coarsePointerQuery.addEventListener("change", onMedia);
      } else if (this.coarsePointerQuery.addListener) {
        this.coarsePointerQuery.addListener(onMedia);
      }
    }
    this.hammer.on("panstart", this.onPanStart);
    this.hammer.on("panmove", this.onPanMove);
    this.hammer.on("panend", this.onPanEnd);
    this.hammer.on("pancancel", this.onPanEnd);
    this.hammer.on("singletap", this.onSingletap);
    this.hammer.on("pinchstart", this.onPinchStart);
    this.hammer.on("pinchmove", this.onPinchMove);
    this.hammer.on("pinchend", this.onPinchEnd);
    this.hammer.on("pinchcancel", this.onPinchEnd);
    this.hammer.on("press", this.onPress);
    this.scrollerEl.addEventListener(
      "contextmenu",
      this.onContextmenu,
      false
    );
    this.scrollerEl.addEventListener("wheel", this.onWheel, false);
    const pageId = this.options.pageId && this.getPageSpreadPositionFromPageId(this.options.pageId) || 0;
    this.hammer.set({ enable: true });
    this.started = true;
    this.destroyed = false;
    this.navigateTo(pageId, { duration: 0 });
    this.el.addEventListener("touchstart", this.onTouchStart, false);
    this.el.addEventListener("touchend", this.onTouchEnd, false);
    if (typeof window !== "undefined") {
      window.addEventListener("resize", this.onResize, false);
      this.prefersReducedMotionQuery = typeof window !== "undefined" && window.matchMedia ? window.matchMedia("(prefers-reduced-motion: reduce)") : null;
    }
    return this;
  }
  destroy() {
    if (!this.started) {
      return console.warn(
        "You've called .destroy() on a viewer that was not started yet, this is a no-op."
      );
    }
    if (this.destroyed) {
      return console.warn(
        "You've called .destroy() on a viewer that has already been destroyed and not restarted, this is a no-op."
      );
    }
    this.scrollerEl.removeEventListener("contextmenu", this.onContextmenu);
    this.scrollerEl.removeEventListener("wheel", this.onWheel);
    this.hammer.destroy();
    this.el.removeEventListener("touchstart", this.onTouchStart);
    this.el.removeEventListener("touchend", this.onTouchEnd);
    if (typeof window !== "undefined") {
      window.removeEventListener("resize", this.onResize);
    }
    this.started = false;
    this.destroyed = true;
    return this;
  }
  first(options) {
    return this.navigateTo(0, options);
  }
  prev(options) {
    return this.navigateTo(this.getPosition() - 1, options);
  }
  next(options) {
    return this.navigateTo(this.getPosition() + 1, options);
  }
  last(options) {
    return this.navigateTo(this.getPageSpreadCount() - 1, options);
  }
  navigateTo(newPosition, options) {
    var _a2, _b, _c;
    if (this.destroyed) {
      return console.warn(" \nYou've called a navigation method on a viewer that was previously destroyed, this is a no-op.\nPlease call viewer.start() again, if you want to reuse this Viewer instance.\n\nYou might have forgotten to remove an event handler that\ncalls first/prev/next/last/navigateTo on the viewer.");
    }
    if (!this.started) {
      return console.warn("\nYou've called a navigation method on a viewer that hasn't been started yet, this is a no-op.\nPlease call viewer.start() first.\n\nYou might have forgotten to remove an event handler that\ncalls .first()/.prev()/.next()/.last()/.navigateTo() on the viewer.\n");
    }
    if (newPosition < 0 || newPosition > this.getPageSpreadCount() - 1) {
      return;
    }
    const currentPosition = this.getPosition();
    const currentPageSpread = this.getPageSpreadFromPosition(currentPosition);
    const activePageSpread = this.getPageSpreadFromPosition(newPosition);
    const touchAction = activePageSpread.isScrollable() ? "pan-y" : "none";
    let carousel = this.getCarouselFromPageSpread(activePageSpread);
    const velocity = (_a2 = options == null ? void 0 : options.velocity) != null ? _a2 : 1;
    let duration = (_b = options == null ? void 0 : options.duration) != null ? _b : this.navigationDuration;
    if ((_c = this.prefersReducedMotionQuery) == null ? void 0 : _c.matches) {
      duration = 0;
    } else if (duration > 0) {
      duration = duration / Math.abs(velocity);
    }
    currentPageSpread == null ? void 0 : currentPageSpread.deactivate();
    activePageSpread.activate();
    carousel.visible.forEach((pageSpread) => {
      pageSpread.position().setVisibility("visible");
    });
    this.hammer.set({ touchAction });
    this.transform.left = this.getLeftTransformFromPageSpread(
      newPosition,
      activePageSpread
    );
    this.setPosition(newPosition);
    if (this.transform.scale > 1) {
      this.transform.top = 0;
      this.transform.scale = 1;
      this.trigger("zoomedOut", { position: currentPosition });
    }
    this.trigger("beforeNavigation", { currentPosition, newPosition });
    this.animation.animate({ x: this.transform.left + "%", duration }, () => {
      carousel = this.getCarouselFromPageSpread(
        this.getActivePageSpread()
      );
      carousel.gone.forEach((pageSpread) => {
        pageSpread.setVisibility("gone");
      });
      this.trigger("afterNavigation", {
        newPosition: this.getPosition(),
        previousPosition: currentPosition
      });
    });
  }
  getPosition() {
    return this.position;
  }
  setPosition(position) {
    this.position = position;
    return this;
  }
  getLeftTransformFromPageSpread(position, pageSpread) {
    if (position === this.getPageSpreadCount() - 1) {
      return 100 - pageSpread.getWidth() - pageSpread.getLeft();
    }
    if (position > 0) {
      return (100 - pageSpread.getWidth()) / 2 - pageSpread.getLeft();
    }
    return 0;
  }
  getCarouselFromPageSpread(pageSpreadSubject) {
    const carousel = {
      visible: [],
      gone: []
    };
    this.pageSpreads.forEach((pageSpread) => {
      let visible = false;
      if (pageSpread.getLeft() <= pageSpreadSubject.getLeft()) {
        if (pageSpread.getLeft() + pageSpread.getWidth() > pageSpreadSubject.getLeft() - 100) {
          visible = true;
        }
      } else {
        if (pageSpread.getLeft() - pageSpread.getWidth() < pageSpreadSubject.getLeft() + 100) {
          visible = true;
        }
      }
      if (visible) {
        carousel.visible.push(pageSpread);
      } else {
        carousel.gone.push(pageSpread);
      }
    });
    return carousel;
  }
  getCoordinateInfo(x, y, pageSpread) {
    x -= this.el.offsetLeft;
    y -= this.el.offsetTop;
    const info = {
      x,
      y,
      contentX: 0,
      contentY: 0,
      pageX: 0,
      pageY: 0,
      overlayEls: [],
      pageEl: null,
      isInsideContentX: false,
      isInsideContentY: false,
      isInsideContent: false
    };
    const overlayEls = pageSpread.getOverlayEls();
    for (let idx = 0; idx < overlayEls.length; idx++) {
      const overlayEl = overlayEls[idx];
      if (isCoordinateInsideElement(x, y, overlayEl)) {
        info.overlayEls.push(overlayEl);
      }
    }
    const pageEls = pageSpread.getPageEls();
    for (let idx = 0; idx < pageEls.length; idx++) {
      const pageEl = pageEls[idx];
      if (isCoordinateInsideElement(x, y, pageEl)) {
        info.pageEl = pageEl;
        break;
      }
    }
    const contentRect = pageSpread.getContentRect();
    info.contentX = (x - contentRect.left) / Math.max(1, contentRect.width);
    info.contentY = (y - contentRect.top) / Math.max(1, contentRect.height);
    if (info.pageEl) {
      info.isInsideContentX = info.contentX >= 0 && info.contentX <= 1;
      info.isInsideContentY = info.contentY >= 0 && info.contentY <= 1;
      info.isInsideContent = info.isInsideContentX && info.isInsideContentY;
    }
    return info;
  }
  getPageSpreadCount() {
    return this.pageSpreads.length;
  }
  getActivePageSpread() {
    return this.getPageSpreadFromPosition(this.getPosition());
  }
  getPageSpreadFromPosition(position) {
    return this.pageSpreads[position];
  }
  getPageSpreadPositionFromPageId(pageId) {
    for (let idx = 0; idx < this.pageSpreads.length; idx++) {
      const pageSpread = this.pageSpreads[idx];
      if (pageSpread.options.pageIds.indexOf(pageId) > -1) return idx;
    }
  }
  zoomTo({
    duration,
    easing,
    scale,
    x = 0,
    y = 0,
    bounds
  } = {}, callback) {
    const curScale = this.transform.scale;
    const activePageSpread = this.getActivePageSpread();
    const { left, top, width, height, pageSpreadRect } = getPageSpreadBounds(activePageSpread);
    const carouselOffset = activePageSpread.getLeft();
    const carouselScaledOffset = carouselOffset * curScale;
    if (scale !== 1) {
      x -= pageSpreadRect.left;
      y -= pageSpreadRect.top;
      x = x / (pageSpreadRect.width / curScale) * 100;
      y = y / (pageSpreadRect.height / curScale) * 100;
      x = this.transform.left + carouselScaledOffset + x - x * scale / curScale;
      y = this.transform.top + y - y * scale / curScale;
      if (bounds !== false && scale > 1) {
        x = clipCoordinate(x, scale, width, left);
        y = clipCoordinate(y, scale, height, top);
      }
    } else {
      x = 0;
      y = 0;
    }
    x -= carouselOffset * scale;
    this.transform.left = x;
    this.transform.top = y;
    this.transform.scale = scale;
    this.animation.animate(
      { x: x + "%", y: y + "%", scale, easing, duration },
      callback
    );
  }
  refresh() {
    this.pageSpreadEls = this.el.querySelectorAll(".verso__page-spread");
    this.pageSpreads = traversePageSpreads(this.pageSpreadEls);
    this.pageIds = buildPageIds(this.pageSpreads);
    return this;
  }
};
export {
  Verso as default
};
//# sourceMappingURL=verso.es.js.map
